@@ -5,33 +5,34 @@
 #ifndef CONTENT_RENDERER_MEDIA_ANDROID_WEBMEDIAPLAYER_ANDROID_H_
 #define CONTENT_RENDERER_MEDIA_ANDROID_WEBMEDIAPLAYER_ANDROID_H_
 
-#include <jni.h>
+#include <stdint.h>
+
+#include <memory>
 #include <string>
 #include <vector>
 
-#include "base/basictypes.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/threading/thread_checker.h"
 #include "base/time/default_tick_clock.h"
 #include "base/time/time.h"
 #include "cc/layers/video_frame_provider.h"
-#include "content/common/media/media_player_messages_enums_android.h"
-#include "content/public/renderer/render_frame_observer.h"
 #include "content/renderer/media/android/media_info_loader.h"
 #include "content/renderer/media/android/media_source_delegate.h"
+#include "content/renderer/media/android/renderer_media_player_manager.h"
 #include "content/renderer/media/android/stream_texture_factory.h"
 #include "gpu/command_buffer/common/mailbox.h"
 #include "media/base/android/media_player_android.h"
 #include "media/base/cdm_context.h"
 #include "media/base/demuxer_stream.h"
+#include "media/base/eme_constants.h"
 #include "media/base/media_keys.h"
 #include "media/base/time_delta_interpolator.h"
-#include "media/blink/webmediaplayer_util.h"
-#include "media/cdm/proxy_decryptor.h"
-#include "third_party/WebKit/public/platform/WebGraphicsContext3D.h"
+#include "media/blink/webmediaplayer_delegate.h"
+#include "media/blink/webmediaplayer_params.h"
 #include "third_party/WebKit/public/platform/WebMediaPlayer.h"
+#include "third_party/WebKit/public/platform/WebSetSinkIdCallbacks.h"
 #include "third_party/WebKit/public/platform/WebSize.h"
 #include "third_party/WebKit/public/platform/WebURL.h"
 #include "third_party/skia/include/core/SkBitmap.h"
@@ -46,6 +47,7 @@ class WebContentDecryptionModule;
 class WebContentDecryptionModuleResult;
 class WebFrame;
 class WebMediaPlayerClient;
+class WebMediaPlayerEncryptedMediaClient;
 class WebURL;
 }
 
@@ -54,126 +56,128 @@ class WebLayerImpl;
 }
 
 namespace gpu {
+namespace gles2 {
+class GLES2Interface;
+}
 struct MailboxHolder;
 }
 
 namespace media {
 class CdmContext;
-class CdmFactory;
 class MediaLog;
-class MediaPermission;
 class WebContentDecryptionModuleImpl;
-class WebMediaPlayerDelegate;
 }
 
 namespace content {
 
-class RendererCdmManager;
 class RendererMediaPlayerManager;
 
 // This class implements blink::WebMediaPlayer by keeping the android
 // media player in the browser process. It listens to all the status changes
 // sent from the browser process and sends playback controls to the media
 // player.
-class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
-                              public cc::VideoFrameProvider,
-                              public RenderFrameObserver,
-                              public StreamTextureFactoryContextObserver {
+class WebMediaPlayerAndroid
+    : public blink::WebMediaPlayer,
+      public cc::VideoFrameProvider,
+      public media::RendererMediaPlayerInterface,
+      public NON_EXPORTED_BASE(media::WebMediaPlayerDelegate::Observer) {
  public:
   // Construct a WebMediaPlayerAndroid object. This class communicates with the
   // MediaPlayerAndroid object in the browser process through |proxy|.
   // TODO(qinmin): |frame| argument is used to determine whether the current
   // player can enter fullscreen. This logic should probably be moved into
-  // blink, so that enterFullscreen() will not be called if another video is
+  // blink, so that enteredFullscreen() will not be called if another video is
   // already in fullscreen.
   WebMediaPlayerAndroid(
       blink::WebFrame* frame,
       blink::WebMediaPlayerClient* client,
+      blink::WebMediaPlayerEncryptedMediaClient* encrypted_client,
       base::WeakPtr<media::WebMediaPlayerDelegate> delegate,
       RendererMediaPlayerManager* player_manager,
-      media::CdmFactory* cdm_factory,
-      media::MediaPermission* media_permission,
-      blink::WebContentDecryptionModule* initial_cdm,
       scoped_refptr<StreamTextureFactory> factory,
-      const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
-      media::MediaLog* media_log);
-  virtual ~WebMediaPlayerAndroid();
+      int frame_id,
+      bool enable_texture_copy,
+      const media::WebMediaPlayerParams& params);
+  ~WebMediaPlayerAndroid() override;
 
   // blink::WebMediaPlayer implementation.
-  virtual void enterFullscreen();
+  bool supportsOverlayFullscreenVideo() override;
+  void enteredFullscreen() override;
 
   // Resource loading.
-  virtual void load(LoadType load_type,
-                    const blink::WebURL& url,
-                    CORSMode cors_mode);
+  void load(LoadType load_type,
+            const blink::WebMediaPlayerSource& source,
+            CORSMode cors_mode) override;
 
   // Playback controls.
-  virtual void play();
-  virtual void pause();
-  virtual void seek(double seconds);
-  virtual bool supportsSave() const;
-  virtual void setRate(double rate);
-  virtual void setVolume(double volume);
-  virtual void setSinkId(const blink::WebString& device_id,
-                         media::WebSetSinkIdCB* raw_web_callbacks);
-  virtual void requestRemotePlayback();
-  virtual void requestRemotePlaybackControl();
-  virtual blink::WebTimeRanges buffered() const;
-  virtual blink::WebTimeRanges seekable() const;
+  void play() override;
+  void pause() override;
+  void seek(double seconds) override;
+  bool supportsSave() const override;
+  void setRate(double rate) override;
+  void setVolume(double volume) override;
+  void setSinkId(const blink::WebString& sink_id,
+                 const blink::WebSecurityOrigin& security_origin,
+                 blink::WebSetSinkIdCallbacks* web_callback) override;
+  void requestRemotePlayback() override;
+  void requestRemotePlaybackControl() override;
+  blink::WebTimeRanges buffered() const override;
+  blink::WebTimeRanges seekable() const override;
 
   // Poster image, as defined in the <video> element.
-  virtual void setPoster(const blink::WebURL& poster) override;
+  void setPoster(const blink::WebURL& poster) override;
 
   // Methods for painting.
   // FIXME: This path "only works" on Android. It is a workaround for the
   // issue that Skia could not handle Android's GL_TEXTURE_EXTERNAL_OES texture
   // internally. It should be removed and replaced by the normal paint path.
   // https://code.google.com/p/skia/issues/detail?id=1189
-  virtual void paint(blink::WebCanvas* canvas,
-                     const blink::WebRect& rect,
-                     unsigned char alpha,
-                     SkXfermode::Mode mode);
+  void paint(blink::WebCanvas* canvas,
+             const blink::WebRect& rect,
+             unsigned char alpha,
+             SkXfermode::Mode mode) override;
 
-  bool copyVideoTextureToPlatformTexture(
-      blink::WebGraphicsContext3D* web_graphics_context,
-      unsigned int texture,
-      unsigned int internal_format,
-      unsigned int type,
-      bool premultiply_alpha,
-      bool flip_y) override;
+  bool copyVideoTextureToPlatformTexture(gpu::gles2::GLES2Interface* gl,
+                                         unsigned int texture,
+                                         unsigned int internal_format,
+                                         unsigned int type,
+                                         bool premultiply_alpha,
+                                         bool flip_y) override;
 
   // True if the loaded media has a playable video/audio track.
-  virtual bool hasVideo() const;
-  virtual bool hasAudio() const;
+  bool hasVideo() const override;
+  bool hasAudio() const override;
 
-  virtual bool isRemote() const;
+  bool isRemote() const override;
 
   // Dimensions of the video.
-  virtual blink::WebSize naturalSize() const;
+  blink::WebSize naturalSize() const override;
 
   // Getters of playback state.
-  virtual bool paused() const;
-  virtual bool seeking() const;
-  virtual double duration() const;
+  bool paused() const override;
+  bool seeking() const override;
+  double duration() const override;
   virtual double timelineOffset() const;
-  virtual double currentTime() const;
+  double currentTime() const override;
 
-  virtual bool didLoadingProgress();
+  bool didLoadingProgress() override;
 
   // Internal states of loading and network.
-  virtual blink::WebMediaPlayer::NetworkState networkState() const;
-  virtual blink::WebMediaPlayer::ReadyState readyState() const;
+  blink::WebMediaPlayer::NetworkState getNetworkState() const override;
+  blink::WebMediaPlayer::ReadyState getReadyState() const override;
 
-  virtual bool hasSingleSecurityOrigin() const;
-  virtual bool didPassCORSAccessCheck() const;
+  blink::WebString getErrorMessage() override;
 
-  virtual double mediaTimeForTimeValue(double timeValue) const;
+  bool hasSingleSecurityOrigin() const override;
+  bool didPassCORSAccessCheck() const override;
+
+  double mediaTimeForTimeValue(double timeValue) const override;
 
   // Provide statistics.
-  virtual unsigned decodedFrameCount() const;
-  virtual unsigned droppedFrameCount() const;
-  virtual unsigned audioDecodedByteCount() const;
-  virtual unsigned videoDecodedByteCount() const;
+  unsigned decodedFrameCount() const override;
+  unsigned droppedFrameCount() const override;
+  size_t audioDecodedByteCount() const override;
+  size_t videoDecodedByteCount() const override;
 
   // cc::VideoFrameProvider implementation. These methods are running on the
   // compositor thread.
@@ -187,85 +191,66 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
 
   // Media player callback handlers.
   void OnMediaMetadataChanged(base::TimeDelta duration, int width,
-                              int height, bool success);
-  void OnPlaybackComplete();
-  void OnBufferingUpdate(int percentage);
-  void OnSeekRequest(const base::TimeDelta& time_to_seek);
-  void OnSeekComplete(const base::TimeDelta& current_time);
-  void OnMediaError(int error_type);
-  void OnVideoSizeChanged(int width, int height);
+                              int height, bool success) override;
+  void OnPlaybackComplete() override;
+  void OnBufferingUpdate(int percentage) override;
+  void OnSeekRequest(const base::TimeDelta& time_to_seek) override;
+  void OnSeekComplete(const base::TimeDelta& current_time) override;
+  void OnMediaError(int error_type) override;
+  void OnVideoSizeChanged(int width, int height) override;
   void OnDurationChanged(const base::TimeDelta& duration);
 
   // Called to update the current time.
   void OnTimeUpdate(base::TimeDelta current_timestamp,
-                    base::TimeTicks current_time_ticks);
+                    base::TimeTicks current_time_ticks) override;
 
   // Functions called when media player status changes.
-  void OnConnectedToRemoteDevice(const std::string& remote_playback_message);
-  void OnDisconnectedFromRemoteDevice();
-  void OnDidExitFullscreen();
-  void OnMediaPlayerPlay();
-  void OnMediaPlayerPause();
-  void OnRemoteRouteAvailabilityChanged(bool routes_available);
-
-  // StreamTextureFactoryContextObserver implementation.
-  void ResetStreamTextureProxy() override;
+  void OnConnectedToRemoteDevice(const std::string& remote_playback_message)
+      override;
+  void OnDisconnectedFromRemoteDevice() override;
+  void OnCancelledRemotePlaybackRequest() override;
+  void OnDidExitFullscreen() override;
+  void OnMediaPlayerPlay() override;
+  void OnMediaPlayerPause() override;
+  void OnRemoteRouteAvailabilityChanged(bool routes_available) override;
 
   // Called when the player is released.
-  virtual void OnPlayerReleased();
+  void OnPlayerReleased() override;
 
   // This function is called by the RendererMediaPlayerManager to pause the
   // video and release the media player and surface texture when we switch tabs.
   // However, the actual GlTexture is not released to keep the video screenshot.
-  virtual void ReleaseMediaResources();
-
-  // RenderFrameObserver implementation.
-  void OnDestruct() override;
+  void SuspendAndReleaseResources() override;
 
 #if defined(VIDEO_HOLE)
   // Calculate the boundary rectangle of the media player (i.e. location and
   // size of the video frame).
   // Returns true if the geometry has been changed since the last call.
-  bool UpdateBoundaryRectangle();
+  bool UpdateBoundaryRectangle() override;
 
-  const gfx::RectF GetBoundaryRectangle();
+  const gfx::RectF GetBoundaryRectangle() override;
 #endif  // defined(VIDEO_HOLE)
 
-  virtual MediaKeyException generateKeyRequest(
-      const blink::WebString& key_system,
-      const unsigned char* init_data,
-      unsigned init_data_length);
-  virtual MediaKeyException addKey(
-      const blink::WebString& key_system,
-      const unsigned char* key,
-      unsigned key_length,
-      const unsigned char* init_data,
-      unsigned init_data_length,
-      const blink::WebString& session_id);
-  virtual MediaKeyException cancelKeyRequest(
-      const blink::WebString& key_system,
-      const blink::WebString& session_id);
-
-  virtual void setContentDecryptionModule(
+  void setContentDecryptionModule(
       blink::WebContentDecryptionModule* cdm,
-      blink::WebContentDecryptionModuleResult result);
-
-  void OnKeyAdded(const std::string& session_id);
-  void OnKeyError(const std::string& session_id,
-                  media::MediaKeys::KeyError error_code,
-                  uint32 system_code);
-  void OnKeyMessage(const std::string& session_id,
-                    const std::vector<uint8>& message,
-                    const GURL& destination_url);
+      blink::WebContentDecryptionModuleResult result) override;
 
   void OnMediaSourceOpened(blink::WebMediaSource* web_media_source);
 
   void OnEncryptedMediaInitData(media::EmeInitDataType init_data_type,
-                                const std::vector<uint8>& init_data);
+                                const std::vector<uint8_t>& init_data);
 
   // Called when a decoder detects that the key needed to decrypt the stream
   // is not available.
-  void OnWaitingForDecryptionKey();
+  void OnWaitingForDecryptionKey() override;
+
+  // WebMediaPlayerDelegate::Observer implementation.
+  void OnHidden() override;
+  void OnShown() override;
+  void OnSuspendRequested(bool must_suspend) override;
+  void OnPlay() override;
+  void OnPause() override;
+  void OnVolumeMultiplierUpdate(double multiplier) override;
 
  protected:
   // Helper method to update the playing state.
@@ -301,33 +286,22 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   bool IsKeySystemSupported(const std::string& key_system);
   bool IsLocalResource();
 
-  // Actually do the work for generateKeyRequest/addKey so they can easily
-  // report results to UMA.
-  MediaKeyException GenerateKeyRequestInternal(const std::string& key_system,
-                                               const unsigned char* init_data,
-                                               unsigned init_data_length);
-  MediaKeyException AddKeyInternal(const std::string& key_system,
-                                   const unsigned char* key,
-                                   unsigned key_length,
-                                   const unsigned char* init_data,
-                                   unsigned init_data_length,
-                                   const std::string& session_id);
-  MediaKeyException CancelKeyRequestInternal(const std::string& key_system,
-                                             const std::string& session_id);
-
   // Called when |cdm_context| is ready.
   void OnCdmContextReady(media::CdmContext* cdm_context);
 
   // Sets the CDM. Should only be called when |is_player_initialized_| is true
-  // and a new non-null |cdm_context_| is available. Fires |cdm_attached_cb_|
-  // with the result after the CDM is attached.
+  // and a new non-null |cdm_context_| is available. Fires |cdm_attached_cb_| on
+  // the main thread with the result after the CDM is attached.
   void SetCdmInternal(const media::CdmAttachedCB& cdm_attached_cb);
 
-  // Requests that this object notifies when a decryptor is ready through the
-  // |decryptor_ready_cb| provided.
-  // If |decryptor_ready_cb| is null, the existing callback will be fired with
+  // Called when the CDM is attached.
+  void OnCdmAttached(const media::CdmAttachedCB& cdm_attached_cb, bool success);
+
+  // Requests that this object notifies when a CDM is ready through the
+  // |cdm_ready_cb| provided.
+  // If |cdm_ready_cb| is null, the existing callback will be fired with
   // NULL immediately and reset.
-  void SetDecryptorReadyCB(const media::DecryptorReadyCB& decryptor_ready_cb);
+  void SetCdmReadyCB(const MediaSourceDelegate::CdmReadyCB& cdm_ready_cb);
 
   // Called when the ContentDecryptionModule has been attached to the
   // pipeline/decoders.
@@ -336,17 +310,29 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
       bool success);
 
   bool IsHLSStream() const;
+  // Report whether the loaded url, after following redirects, points to a HLS
+  // playlist, and record the origin of the player.
+  void ReportHLSMetrics() const;
+
+  // Called after |defer_load_cb_| has decided to allow the load. If
+  // |defer_load_cb_| is null this is called immediately.
+  void DoLoad(LoadType load_type, const blink::WebURL& url, CORSMode cors_mode);
 
   blink::WebFrame* const frame_;
 
   blink::WebMediaPlayerClient* const client_;
+  blink::WebMediaPlayerEncryptedMediaClient* const encrypted_client_;
 
-  // |delegate_| is used to notify the browser process of the player status, so
-  // that the browser process can control screen locks.
-  // TODO(qinmin): Currently android mediaplayer takes care of the screen
-  // lock. So this is only used for media source. Will apply this to regular
-  // media tag once http://crbug.com/247892 is fixed.
+  // WebMediaPlayer notifies the |delegate_| of playback state changes using
+  // |delegate_id_|; an id provided after registering with the delegate.  The
+  // WebMediaPlayer may also receive directives (play, pause) from the delegate
+  // via the WebMediaPlayerDelegate::Observer interface after registration.
   base::WeakPtr<media::WebMediaPlayerDelegate> delegate_;
+  int delegate_id_;
+
+  // Callback responsible for determining if loading of media should be deferred
+  // for external reasons; called during load().
+  media::WebMediaPlayerParams::DeferLoadCB defer_load_cb_;
 
   // Save the list of buffered time ranges.
   blink::WebTimeRanges buffered_;
@@ -354,13 +340,16 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // Size of the video.
   blink::WebSize natural_size_;
 
-  // Size that has been sent to StreamTexture.
+  // Size that has been sent to gpu::StreamTexture.
   blink::WebSize cached_stream_texture_size_;
 
   // The video frame object used for rendering by the compositor.
   scoped_refptr<media::VideoFrame> current_frame_;
   base::Lock current_frame_lock_;
 
+  // A lazily created transparent video frame to be displayed in fullscreen.
+  scoped_refptr<media::VideoFrame> fullscreen_frame_;
+
   base::ThreadChecker main_thread_checker_;
 
   // Message loop for media thread.
@@ -397,15 +386,16 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // Owned by RenderFrameImpl.
   RendererMediaPlayerManager* const player_manager_;
 
-  // TODO(xhwang): Remove |cdm_factory_| when prefixed EME is deprecated. See
-  // http://crbug.com/249976
-  media::CdmFactory* const cdm_factory_;
-
-  media::MediaPermission* media_permission_;
-
   // Player ID assigned by the |player_manager_|.
   int player_id_;
 
+  // User created media session id, if any.
+  //
+  // blink::WebMediaSession::DefaultID represents the non web
+  // exposed default media session. User created session ids are
+  // greater than blink::WebMediaSession::DefaultID.
+  const int media_session_id_;
+
   // Current player states.
   blink::WebMediaPlayer::NetworkState network_state_;
   blink::WebMediaPlayer::ReadyState ready_state_;
@@ -426,13 +416,16 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // Whether the media player is playing.
   bool is_playing_;
 
+  // Whether the media player is pending to play.
+  bool is_play_pending_;
+
   // Whether media player needs to re-establish the surface texture peer.
   bool needs_establish_peer_;
 
   // Whether the video size info is available.
   bool has_size_info_;
 
-  const scoped_refptr<base::SingleThreadTaskRunner> compositor_loop_;
+  const scoped_refptr<base::SingleThreadTaskRunner> compositor_task_runner_;
 
   // Object for allocating stream textures.
   scoped_refptr<StreamTextureFactory> stream_texture_factory_;
@@ -456,7 +449,7 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // blocked.
   cc::VideoFrameProvider::Client* video_frame_provider_client_;
 
-  scoped_ptr<cc_blink::WebLayerImpl> video_weblayer_;
+  std::unique_ptr<cc_blink::WebLayerImpl> video_weblayer_;
 
 #if defined(VIDEO_HOLE)
   // A rectangle represents the geometry of video frame, when computed last
@@ -475,18 +468,7 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
 
   scoped_refptr<media::MediaLog> media_log_;
 
-  scoped_ptr<MediaInfoLoader> info_loader_;
-
-  // The currently selected key system. Empty string means that no key system
-  // has been selected.
-  std::string current_key_system_;
-
-  // Temporary for EME v0.1. Not needed for unprefixed EME, and can be removed
-  // when prefixed EME is removed.
-  media::EmeInitDataType init_data_type_;
-
-  // Manages decryption keys and decrypts encrypted frames.
-  scoped_ptr<media::ProxyDecryptor> proxy_decryptor_;
+  std::unique_ptr<MediaInfoLoader> info_loader_;
 
   // Non-owned pointer to the CdmContext. Updated in the constructor,
   // generateKeyRequest() or setContentDecryptionModule().
@@ -496,7 +478,7 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // side CDM will be used. This is similar to WebMediaPlayerImpl. For other key
   // systems, a browser side CDM will be used and we set CDM by calling
   // player_manager_->SetCdm() directly.
-  media::DecryptorReadyCB decryptor_ready_cb_;
+  MediaSourceDelegate::CdmReadyCB cdm_ready_cb_;
 
   SkBitmap bitmap_;
 
@@ -513,7 +495,26 @@ class WebMediaPlayerAndroid : public blink::WebMediaPlayer,
   // as playback progresses.
   media::TimeDeltaInterpolator interpolator_;
 
-  scoped_ptr<MediaSourceDelegate> media_source_delegate_;
+  std::unique_ptr<MediaSourceDelegate> media_source_delegate_;
+
+  int frame_id_;
+
+  // Whether to require that surface textures are copied in order to support
+  // sharing between render and gpu threads in WebView.
+  bool enable_texture_copy_;
+
+  // Whether to delete the existing texture and re-create it.
+  bool suppress_deleting_texture_;
+
+  // Whether OnPlaybackComplete() has been called since the last playback.
+  bool playback_completed_;
+
+  // The last volume received by setVolume() and the last volume multiplier from
+  // OnVolumeMultiplierUpdate().  The multiplier is typical 1.0, but may be less
+  // if the WebMediaPlayerDelegate has requested a volume reduction (ducking)
+  // for a transient sound.  Playout volume is derived by volume * multiplier.
+  double volume_;
+  double volume_multiplier_;
 
   // NOTE: Weak pointers must be invalidated before all other member variables.
   base::WeakPtrFactory<WebMediaPlayerAndroid> weak_factory_;
