@@ -4,17 +4,18 @@
 
 #include "net/websockets/websocket_basic_handshake_stream.h"
 
+#include <stddef.h>
 #include <algorithm>
 #include <iterator>
 #include <set>
 #include <string>
+#include <unordered_set>
+#include <utility>
 #include <vector>
 
 #include "base/base64.h"
-#include "base/basictypes.h"
 #include "base/bind.h"
 #include "base/compiler_specific.h"
-#include "base/containers/hash_tables.h"
 #include "base/logging.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/sparse_histogram.h"
@@ -35,6 +36,7 @@
 #include "net/socket/client_socket_handle.h"
 #include "net/socket/websocket_transport_client_socket_pool.h"
 #include "net/websockets/websocket_basic_stream.h"
+#include "net/websockets/websocket_deflate_parameters.h"
 #include "net/websockets/websocket_deflate_predictor.h"
 #include "net/websockets/websocket_deflate_predictor_impl.h"
 #include "net/websockets/websocket_deflate_stream.h"
@@ -57,14 +59,8 @@ const char kConnectionErrorStatusLine[] = "HTTP/1.1 503 Connection Error";
 // TODO(ricea): If more extensions are added, replace this with a more general
 // mechanism.
 struct WebSocketExtensionParams {
-  WebSocketExtensionParams()
-      : deflate_enabled(false),
-        client_window_bits(15),
-        deflate_mode(WebSocketDeflater::TAKE_OVER_CONTEXT) {}
-
-  bool deflate_enabled;
-  int client_window_bits;
-  WebSocketDeflater::ContextTakeOverMode deflate_mode;
+  bool deflate_enabled = false;
+  WebSocketDeflateParameters deflate_parameters;
 };
 
 namespace {
@@ -99,16 +95,16 @@ void AddVectorHeaderIfNonEmpty(const char* name,
                                HttpRequestHeaders* headers) {
   if (value.empty())
     return;
-  headers->SetHeader(name, JoinString(value, ", "));
+  headers->SetHeader(name, base::JoinString(value, ", "));
 }
 
 GetHeaderResult GetSingleHeaderValue(const HttpResponseHeaders* headers,
                                      const base::StringPiece& name,
                                      std::string* value) {
-  void* state = nullptr;
+  size_t iter = 0;
   size_t num_values = 0;
   std::string temp_value;
-  while (headers->EnumerateHeader(&state, name, &temp_value)) {
+  while (headers->EnumerateHeader(&iter, name, &temp_value)) {
     if (++num_values > 1)
       return GET_HEADER_MULTIPLE;
     *value = temp_value;
@@ -189,18 +185,18 @@ bool ValidateSubProtocol(
     const std::vector<std::string>& requested_sub_protocols,
     std::string* sub_protocol,
     std::string* failure_message) {
-  void* state = nullptr;
+  size_t iter = 0;
   std::string value;
-  base::hash_set<std::string> requested_set(requested_sub_protocols.begin(),
-                                            requested_sub_protocols.end());
+  std::unordered_set<std::string> requested_set(requested_sub_protocols.begin(),
+                                                requested_sub_protocols.end());
   int count = 0;
   bool has_multiple_protocols = false;
   bool has_invalid_protocol = false;
 
   while (!has_invalid_protocol || !has_multiple_protocols) {
     std::string temp_value;
-    if (!headers->EnumerateHeader(
-            &state, websockets::kSecWebSocketProtocol, &temp_value))
+    if (!headers->EnumerateHeader(&iter, websockets::kSecWebSocketProtocol,
+                                  &temp_value))
       break;
     value = temp_value;
     if (requested_set.count(value) == 0)
@@ -235,84 +231,17 @@ bool ValidateSubProtocol(
   return true;
 }
 
-bool DeflateError(std::string* message, const base::StringPiece& piece) {
-  *message = "Error in permessage-deflate: ";
-  piece.AppendToString(message);
-  return false;
-}
-
-bool ValidatePerMessageDeflateExtension(const WebSocketExtension& extension,
-                                        std::string* failure_message,
-                                        WebSocketExtensionParams* params) {
-  static const char kClientPrefix[] = "client_";
-  static const char kServerPrefix[] = "server_";
-  static const char kNoContextTakeover[] = "no_context_takeover";
-  static const char kMaxWindowBits[] = "max_window_bits";
-  const size_t kPrefixLen = arraysize(kClientPrefix) - 1;
-  static_assert(kPrefixLen == arraysize(kServerPrefix) - 1,
-                "the strings server and client must be the same length");
-  typedef std::vector<WebSocketExtension::Parameter> ParameterVector;
-
-  DCHECK_EQ("permessage-deflate", extension.name());
-  const ParameterVector& parameters = extension.parameters();
-  std::set<std::string> seen_names;
-  for (ParameterVector::const_iterator it = parameters.begin();
-       it != parameters.end(); ++it) {
-    const std::string& name = it->name();
-    if (seen_names.count(name) != 0) {
-      return DeflateError(
-          failure_message,
-          "Received duplicate permessage-deflate extension parameter " + name);
-    }
-    seen_names.insert(name);
-    const std::string client_or_server(name, 0, kPrefixLen);
-    const bool is_client = (client_or_server == kClientPrefix);
-    if (!is_client && client_or_server != kServerPrefix) {
-      return DeflateError(
-          failure_message,
-          "Received an unexpected permessage-deflate extension parameter");
-    }
-    const std::string rest(name, kPrefixLen);
-    if (rest == kNoContextTakeover) {
-      if (it->HasValue()) {
-        return DeflateError(failure_message,
-                            "Received invalid " + name + " parameter");
-      }
-      if (is_client)
-        params->deflate_mode = WebSocketDeflater::DO_NOT_TAKE_OVER_CONTEXT;
-    } else if (rest == kMaxWindowBits) {
-      if (!it->HasValue())
-        return DeflateError(failure_message, name + " must have value");
-      int bits = 0;
-      if (!base::StringToInt(it->value(), &bits) || bits < 8 || bits > 15 ||
-          it->value()[0] == '0' ||
-          it->value().find_first_not_of("0123456789") != std::string::npos) {
-        return DeflateError(failure_message,
-                            "Received invalid " + name + " parameter");
-      }
-      if (is_client)
-        params->client_window_bits = bits;
-    } else {
-      return DeflateError(
-          failure_message,
-          "Received an unexpected permessage-deflate extension parameter");
-    }
-  }
-  params->deflate_enabled = true;
-  return true;
-}
-
 bool ValidateExtensions(const HttpResponseHeaders* headers,
                         std::string* accepted_extensions_descriptor,
                         std::string* failure_message,
                         WebSocketExtensionParams* params) {
-  void* state = nullptr;
+  size_t iter = 0;
   std::string header_value;
   std::vector<std::string> header_values;
   // TODO(ricea): If adding support for additional extensions, generalise this
   // code.
   bool seen_permessage_deflate = false;
-  while (headers->EnumerateHeader(&state, websockets::kSecWebSocketExtensions,
+  while (headers->EnumerateHeader(&iter, websockets::kSecWebSocketExtensions,
                                   &header_value)) {
     WebSocketExtensionParser parser;
     if (!parser.Parse(header_value)) {
@@ -332,11 +261,16 @@ bool ValidateExtensions(const HttpResponseHeaders* headers,
           return false;
         }
         seen_permessage_deflate = true;
-
-        if (!ValidatePerMessageDeflateExtension(extension, failure_message,
-                                                params)) {
+        auto& deflate_parameters = params->deflate_parameters;
+        if (!deflate_parameters.Initialize(extension, failure_message) ||
+            !deflate_parameters.IsValidAsResponse(failure_message)) {
+          *failure_message = "Error in permessage-deflate: " + *failure_message;
           return false;
         }
+        // Note that we don't have to check the request-response compatibility
+        // here because we send a request compatible with any valid responses.
+        // TODO(yhirano): Place a DCHECK here.
+
         header_values.push_back(header_value);
       } else {
         *failure_message = "Found an unsupported extension '" +
@@ -346,14 +280,15 @@ bool ValidateExtensions(const HttpResponseHeaders* headers,
       }
     }
   }
-  *accepted_extensions_descriptor = JoinString(header_values, ", ");
+  *accepted_extensions_descriptor = base::JoinString(header_values, ", ");
+  params->deflate_enabled = seen_permessage_deflate;
   return true;
 }
 
 }  // namespace
 
 WebSocketBasicHandshakeStream::WebSocketBasicHandshakeStream(
-    scoped_ptr<ClientSocketHandle> connection,
+    std::unique_ptr<ClientSocketHandle> connection,
     WebSocketStream::ConnectDelegate* connect_delegate,
     bool using_proxy,
     std::vector<std::string> requested_sub_protocols,
@@ -420,10 +355,10 @@ int WebSocketBasicHandshakeStream::SendRequest(
       ComputeSecWebSocketAccept(handshake_challenge);
 
   DCHECK(connect_delegate_);
-  scoped_ptr<WebSocketHandshakeRequestInfo> request(
+  std::unique_ptr<WebSocketHandshakeRequestInfo> request(
       new WebSocketHandshakeRequestInfo(url_, base::Time::Now()));
   request->headers.CopyFrom(enriched_headers);
-  connect_delegate_->OnStartOpeningHandshake(request.Pass());
+  connect_delegate_->OnStartOpeningHandshake(std::move(request));
 
   return parser()->SendRequest(
       state_.GenerateRequestLine(), enriched_headers, response, callback);
@@ -462,10 +397,6 @@ bool WebSocketBasicHandshakeStream::IsResponseBodyComplete() const {
   return parser()->IsResponseBodyComplete();
 }
 
-bool WebSocketBasicHandshakeStream::CanFindEndOfResponse() const {
-  return parser() && parser()->CanFindEndOfResponse();
-}
-
 bool WebSocketBasicHandshakeStream::IsConnectionReused() const {
   return parser()->IsConnectionReused();
 }
@@ -474,11 +405,15 @@ void WebSocketBasicHandshakeStream::SetConnectionReused() {
   parser()->SetConnectionReused();
 }
 
-bool WebSocketBasicHandshakeStream::IsConnectionReusable() const {
+bool WebSocketBasicHandshakeStream::CanReuseConnection() const {
   return false;
 }
 
-int64 WebSocketBasicHandshakeStream::GetTotalReceivedBytes() const {
+int64_t WebSocketBasicHandshakeStream::GetTotalReceivedBytes() const {
+  return 0;
+}
+
+int64_t WebSocketBasicHandshakeStream::GetTotalSentBytes() const {
   return 0;
 }
 
@@ -497,7 +432,24 @@ void WebSocketBasicHandshakeStream::GetSSLCertRequestInfo(
   parser()->GetSSLCertRequestInfo(cert_request_info);
 }
 
-bool WebSocketBasicHandshakeStream::IsSpdyHttpStream() const { return false; }
+bool WebSocketBasicHandshakeStream::GetRemoteEndpoint(IPEndPoint* endpoint) {
+  if (!state_.connection() || !state_.connection()->socket())
+    return false;
+
+  return state_.connection()->socket()->GetPeerAddress(endpoint) == OK;
+}
+
+void WebSocketBasicHandshakeStream::PopulateNetErrorDetails(
+    NetErrorDetails* /*details*/) {
+  return;
+}
+
+Error WebSocketBasicHandshakeStream::GetSignedEKMForTokenBinding(
+    crypto::ECPrivateKey* key,
+    std::vector<uint8_t>* out) {
+  NOTREACHED();
+  return ERR_NOT_IMPLEMENTED;
+}
 
 void WebSocketBasicHandshakeStream::Drain(HttpNetworkSession* session) {
   HttpResponseBodyDrainer* drainer = new HttpResponseBodyDrainer(this);
@@ -519,31 +471,27 @@ HttpStream* WebSocketBasicHandshakeStream::RenewStreamForAuth() {
   return nullptr;
 }
 
-scoped_ptr<WebSocketStream> WebSocketBasicHandshakeStream::Upgrade() {
+std::unique_ptr<WebSocketStream> WebSocketBasicHandshakeStream::Upgrade() {
   // The HttpStreamParser object has a pointer to our ClientSocketHandle. Make
   // sure it does not touch it again before it is destroyed.
   state_.DeleteParser();
   WebSocketTransportClientSocketPool::UnlockEndpoint(state_.connection());
-  scoped_ptr<WebSocketStream> basic_stream(
-      new WebSocketBasicStream(state_.ReleaseConnection(),
-                               state_.read_buf(),
-                               sub_protocol_,
-                               extensions_));
+  std::unique_ptr<WebSocketStream> basic_stream(
+      new WebSocketBasicStream(state_.ReleaseConnection(), state_.read_buf(),
+                               sub_protocol_, extensions_));
   DCHECK(extension_params_.get());
   if (extension_params_->deflate_enabled) {
     UMA_HISTOGRAM_ENUMERATION(
         "Net.WebSocket.DeflateMode",
-        extension_params_->deflate_mode,
+        extension_params_->deflate_parameters.client_context_take_over_mode(),
         WebSocketDeflater::NUM_CONTEXT_TAKEOVER_MODE_TYPES);
 
-    return scoped_ptr<WebSocketStream>(
-        new WebSocketDeflateStream(basic_stream.Pass(),
-                                   extension_params_->deflate_mode,
-                                   extension_params_->client_window_bits,
-                                   scoped_ptr<WebSocketDeflatePredictor>(
-                                       new WebSocketDeflatePredictorImpl)));
+    return std::unique_ptr<WebSocketStream>(new WebSocketDeflateStream(
+        std::move(basic_stream), extension_params_->deflate_parameters,
+        std::unique_ptr<WebSocketDeflatePredictor>(
+            new WebSocketDeflatePredictorImpl)));
   } else {
-    return basic_stream.Pass();
+    return basic_stream;
   }
 }
 
