@@ -4,26 +4,30 @@
 
 #include "ui/views/widget/widget.h"
 
-#include "base/debug/trace_event.h"
 #include "base/logging.h"
+#include "base/macros.h"
 #include "base/message_loop/message_loop.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/trace_event/trace_event.h"
 #include "ui/base/cursor/cursor.h"
+#include "ui/base/default_style.h"
 #include "ui/base/default_theme_provider.h"
 #include "ui/base/hit_test.h"
+#include "ui/base/ime/input_method.h"
 #include "ui/base/l10n/l10n_font_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/compositor/compositor.h"
 #include "ui/compositor/layer.h"
+#include "ui/display/screen.h"
 #include "ui/events/event.h"
+#include "ui/events/event_utils.h"
 #include "ui/gfx/image/image_skia.h"
-#include "ui/gfx/screen.h"
 #include "ui/views/controls/menu/menu_controller.h"
+#include "ui/views/event_monitor.h"
 #include "ui/views/focus/focus_manager.h"
 #include "ui/views/focus/focus_manager_factory.h"
 #include "ui/views/focus/view_storage.h"
 #include "ui/views/focus/widget_focus_manager.h"
-#include "ui/views/ime/input_method.h"
 #include "ui/views/views_delegate.h"
 #include "ui/views/widget/native_widget_private.h"
 #include "ui/views/widget/root_view.h"
@@ -54,13 +58,27 @@ void BuildRootLayers(View* view, std::vector<ui::Layer*>* layers) {
 // Create a native widget implementation.
 // First, use the supplied one if non-NULL.
 // Finally, make a default one.
-NativeWidget* CreateNativeWidget(NativeWidget* native_widget,
+NativeWidget* CreateNativeWidget(const Widget::InitParams& params,
                                  internal::NativeWidgetDelegate* delegate) {
-  if (!native_widget) {
-    native_widget =
-        internal::NativeWidgetPrivate::CreateNativeWidget(delegate);
+  if (params.native_widget)
+    return params.native_widget;
+
+  ViewsDelegate* views_delegate = ViewsDelegate::GetInstance();
+  if (views_delegate && !views_delegate->native_widget_factory().is_null()) {
+    NativeWidget* native_widget =
+        views_delegate->native_widget_factory().Run(params, delegate);
+    if (native_widget)
+      return native_widget;
   }
-  return native_widget;
+  return internal::NativeWidgetPrivate::CreateNativeWidget(delegate);
+}
+
+void NotifyCaretBoundsChanged(ui::InputMethod* input_method) {
+  if (!input_method)
+    return;
+  ui::TextInputClient* client = input_method->GetTextInputClient();
+  if (client)
+    input_method->OnCaretBoundsChanged(client);
 }
 
 }  // namespace
@@ -71,19 +89,13 @@ class DefaultWidgetDelegate : public WidgetDelegate {
  public:
   explicit DefaultWidgetDelegate(Widget* widget) : widget_(widget) {
   }
-  virtual ~DefaultWidgetDelegate() {}
+  ~DefaultWidgetDelegate() override {}
 
   // Overridden from WidgetDelegate:
-  virtual void DeleteDelegate() OVERRIDE {
-    delete this;
-  }
-  virtual Widget* GetWidget() OVERRIDE {
-    return widget_;
-  }
-  virtual const Widget* GetWidget() const OVERRIDE {
-    return widget_;
-  }
-  virtual bool ShouldAdvanceFocusToTopLevelWidget() const OVERRIDE {
+  void DeleteDelegate() override { delete this; }
+  Widget* GetWidget() override { return widget_; }
+  const Widget* GetWidget() const override { return widget_; }
+  bool ShouldAdvanceFocusToTopLevelWidget() const override {
     // In most situations where a Widget is used without a delegate the Widget
     // is used as a container, so that we want focus to advance to the top-level
     // widget. A good example of this is the find bar.
@@ -99,33 +111,11 @@ class DefaultWidgetDelegate : public WidgetDelegate {
 ////////////////////////////////////////////////////////////////////////////////
 // Widget, InitParams:
 
-Widget::InitParams::InitParams()
-    : type(TYPE_WINDOW),
-      delegate(NULL),
-      child(false),
-      opacity(INFER_OPACITY),
-      accept_events(true),
-      activatable(ACTIVATABLE_DEFAULT),
-      keep_on_top(false),
-      visible_on_all_workspaces(false),
-      ownership(NATIVE_WIDGET_OWNS_WIDGET),
-      mirror_origin_in_rtl(false),
-      shadow_type(SHADOW_TYPE_DEFAULT),
-      remove_standard_frame(false),
-      use_system_default_icon(false),
-      show_state(ui::SHOW_STATE_DEFAULT),
-      double_buffer(false),
-      parent(NULL),
-      native_widget(NULL),
-      desktop_window_tree_host(NULL),
-      layer_type(aura::WINDOW_LAYER_TEXTURED),
-      context(NULL),
-      force_show_in_taskbar(false) {
-}
+Widget::InitParams::InitParams() : InitParams(TYPE_WINDOW) {}
 
 Widget::InitParams::InitParams(Type type)
     : type(type),
-      delegate(NULL),
+      delegate(nullptr),
       child(false),
       opacity(INFER_OPACITY),
       accept_events(true),
@@ -138,14 +128,15 @@ Widget::InitParams::InitParams(Type type)
       remove_standard_frame(false),
       use_system_default_icon(false),
       show_state(ui::SHOW_STATE_DEFAULT),
-      double_buffer(false),
-      parent(NULL),
-      native_widget(NULL),
-      desktop_window_tree_host(NULL),
-      layer_type(aura::WINDOW_LAYER_TEXTURED),
-      context(NULL),
-      force_show_in_taskbar(false) {
-}
+      parent(nullptr),
+      native_widget(nullptr),
+      desktop_window_tree_host(nullptr),
+      layer_type(ui::LAYER_TEXTURED),
+      context(nullptr),
+      force_show_in_taskbar(false),
+      force_software_compositing(false) {}
+
+Widget::InitParams::InitParams(const InitParams& other) = default;
 
 Widget::InitParams::~InitParams() {
 }
@@ -154,14 +145,14 @@ Widget::InitParams::~InitParams() {
 // Widget, public:
 
 Widget::Widget()
-    : native_widget_(NULL),
-      widget_delegate_(NULL),
-      non_client_view_(NULL),
-      dragged_view_(NULL),
+    : native_widget_(nullptr),
+      widget_delegate_(nullptr),
+      non_client_view_(nullptr),
+      dragged_view_(nullptr),
       ownership_(InitParams::NATIVE_WIDGET_OWNS_WIDGET),
       is_secondary_widget_(true),
       frame_type_(FRAME_TYPE_DEFAULT),
-      disable_inactive_rendering_(false),
+      always_render_as_active_(false),
       widget_closed_(false),
       saved_show_state_(ui::SHOW_STATE_DEFAULT),
       focus_on_creation_(true),
@@ -169,7 +160,7 @@ Widget::Widget()
       native_widget_initialized_(false),
       native_widget_destroyed_(false),
       is_mouse_button_pressed_(false),
-      is_touch_down_(false),
+      ignore_capture_loss_(false),
       last_mouse_event_was_move_(false),
       auto_release_capture_(true),
       root_layers_dirty_(false),
@@ -188,22 +179,6 @@ Widget::~Widget() {
   }
 }
 
-// static
-Widget* Widget::CreateWindow(WidgetDelegate* delegate) {
-  return CreateWindowWithBounds(delegate, gfx::Rect());
-}
-
-// static
-Widget* Widget::CreateWindowWithBounds(WidgetDelegate* delegate,
-                                       const gfx::Rect& bounds) {
-  Widget* widget = new Widget;
-  Widget::InitParams params;
-  params.bounds = bounds;
-  params.delegate = delegate;
-  widget->Init(params);
-  return widget;
-}
-
 // static
 Widget* Widget::CreateWindowWithParent(WidgetDelegate* delegate,
                                        gfx::NativeView parent) {
@@ -225,13 +200,13 @@ Widget* Widget::CreateWindowWithParentAndBounds(WidgetDelegate* delegate,
 
 // static
 Widget* Widget::CreateWindowWithContext(WidgetDelegate* delegate,
-                                        gfx::NativeView context) {
+                                        gfx::NativeWindow context) {
   return CreateWindowWithContextAndBounds(delegate, context, gfx::Rect());
 }
 
 // static
 Widget* Widget::CreateWindowWithContextAndBounds(WidgetDelegate* delegate,
-                                                 gfx::NativeView context,
+                                                 gfx::NativeWindow context,
                                                  const gfx::Rect& bounds) {
   Widget* widget = new Widget;
   Widget::InitParams params;
@@ -264,7 +239,6 @@ Widget* Widget::GetTopLevelWidgetForNativeView(gfx::NativeView native_view) {
   return native_widget ? native_widget->GetWidget() : NULL;
 }
 
-
 // static
 void Widget::GetAllChildWidgets(gfx::NativeView native_view,
                                 Widgets* children) {
@@ -285,14 +259,16 @@ void Widget::ReparentNativeView(gfx::NativeView native_view,
 
 // static
 int Widget::GetLocalizedContentsWidth(int col_resource_id) {
-  return ui::GetLocalizedContentsWidthForFont(col_resource_id,
-      ResourceBundle::GetSharedInstance().GetFont(ResourceBundle::BaseFont));
+  return ui::GetLocalizedContentsWidthForFont(
+      col_resource_id, ResourceBundle::GetSharedInstance().GetFontWithDelta(
+                           ui::kMessageFontSizeDelta));
 }
 
 // static
 int Widget::GetLocalizedContentsHeight(int row_resource_id) {
-  return ui::GetLocalizedContentsHeightForFont(row_resource_id,
-      ResourceBundle::GetSharedInstance().GetFont(ResourceBundle::BaseFont));
+  return ui::GetLocalizedContentsHeightForFont(
+      row_resource_id, ResourceBundle::GetSharedInstance().GetFontWithDelta(
+                           ui::kMessageFontSizeDelta));
 }
 
 // static
@@ -313,6 +289,12 @@ void Widget::Init(const InitParams& in_params) {
   TRACE_EVENT0("views", "Widget::Init");
   InitParams params = in_params;
 
+  // If an internal name was not provided the class name of the contents view
+  // is a reasonable default.
+  if (params.name.empty() && params.delegate &&
+      params.delegate->GetContentsView())
+    params.name = params.delegate->GetContentsView()->GetClassName();
+
   params.child |= (params.type == InitParams::TYPE_CONTROL);
   is_top_level_ = !params.child;
 
@@ -321,8 +303,8 @@ void Widget::Init(const InitParams& in_params) {
       params.type != views::Widget::InitParams::TYPE_PANEL)
     params.opacity = views::Widget::InitParams::OPAQUE_WINDOW;
 
-  if (ViewsDelegate::views_delegate)
-    ViewsDelegate::views_delegate->OnBeforeWidgetInit(&params, this);
+  if (ViewsDelegate::GetInstance())
+    ViewsDelegate::GetInstance()->OnBeforeWidgetInit(&params, this);
 
   if (params.opacity == views::Widget::InitParams::INFER_OPACITY)
     params.opacity = views::Widget::InitParams::OPAQUE_WINDOW;
@@ -347,8 +329,7 @@ void Widget::Init(const InitParams& in_params) {
   widget_delegate_->set_can_activate(can_activate);
 
   ownership_ = params.ownership;
-  native_widget_ = CreateNativeWidget(params.native_widget, this)->
-                   AsNativeWidgetPrivate();
+  native_widget_ = CreateNativeWidget(params, this)->AsNativeWidgetPrivate();
   root_view_.reset(CreateRootView());
   default_theme_provider_.reset(new ui::DefaultThemeProvider);
   if (params.type == InitParams::TYPE_MENU) {
@@ -364,15 +345,19 @@ void Widget::Init(const InitParams& in_params) {
     non_client_view_->set_client_view(widget_delegate_->CreateClientView(this));
     non_client_view_->SetOverlayView(widget_delegate_->CreateOverlayView());
     SetContentsView(non_client_view_);
-    // Initialize the window's title before setting the window's initial bounds;
-    // the frame view's preferred height may depend on the presence of a title.
+    // Initialize the window's icon and title before setting the window's
+    // initial bounds; the frame view's preferred height may depend on the
+    // presence of an icon or a title.
+    UpdateWindowIcon();
     UpdateWindowTitle();
     non_client_view_->ResetWindowControls();
     SetInitialBounds(params.bounds);
-    if (params.show_state == ui::SHOW_STATE_MAXIMIZED)
+    if (params.show_state == ui::SHOW_STATE_MAXIMIZED) {
       Maximize();
-    else if (params.show_state == ui::SHOW_STATE_MINIMIZED)
+    } else if (params.show_state == ui::SHOW_STATE_MINIMIZED) {
       Minimize();
+      saved_show_state_ = ui::SHOW_STATE_MINIMIZED;
+    }
   } else if (params.delegate) {
     SetContentsView(params.delegate->GetContentsView());
     SetInitialBoundsForFramelessWindow(params.bounds);
@@ -381,6 +366,7 @@ void Widget::Init(const InitParams& in_params) {
   // the correct NativeTheme (on Linux). See http://crbug.com/384492
   observer_manager_.Add(GetNativeTheme());
   native_widget_initialized_ = true;
+  native_widget_->OnWidgetInitDone();
 }
 
 // Unconverted methods (see header) --------------------------------------------
@@ -394,6 +380,8 @@ gfx::NativeWindow Widget::GetNativeWindow() const {
 }
 
 void Widget::AddObserver(WidgetObserver* observer) {
+  // Make sure that there is no nullptr in observer list. crbug.com/471649.
+  CHECK(observer);
   observers_.AddObserver(observer);
 }
 
@@ -401,7 +389,7 @@ void Widget::RemoveObserver(WidgetObserver* observer) {
   observers_.RemoveObserver(observer);
 }
 
-bool Widget::HasObserver(WidgetObserver* observer) {
+bool Widget::HasObserver(const WidgetObserver* observer) const {
   return observers_.HasObserver(observer);
 }
 
@@ -413,7 +401,7 @@ void Widget::RemoveRemovalsObserver(WidgetRemovalsObserver* observer) {
   removals_observers_.RemoveObserver(observer);
 }
 
-bool Widget::HasRemovalsObserver(WidgetRemovalsObserver* observer) {
+bool Widget::HasRemovalsObserver(const WidgetRemovalsObserver* observer) const {
   return removals_observers_.HasObserver(observer);
 }
 
@@ -447,9 +435,9 @@ void Widget::NotifyNativeViewHierarchyChanged() {
 }
 
 void Widget::NotifyWillRemoveView(View* view) {
-    FOR_EACH_OBSERVER(WidgetRemovalsObserver,
-                      removals_observers_,
-                      OnWillRemoveView(this, view));
+  FOR_EACH_OBSERVER(WidgetRemovalsObserver,
+                    removals_observers_,
+                    OnWillRemoveView(this, view));
 }
 
 // Converted methods (see header) ----------------------------------------------
@@ -500,6 +488,10 @@ gfx::Rect Widget::GetRestoredBounds() const {
   return native_widget_->GetRestoredBounds();
 }
 
+std::string Widget::GetWorkspace() const {
+  return native_widget_->GetWorkspace();
+}
+
 void Widget::SetBounds(const gfx::Rect& bounds) {
   native_widget_->SetBounds(bounds);
 }
@@ -513,9 +505,9 @@ void Widget::CenterWindow(const gfx::Size& size) {
 }
 
 void Widget::SetBoundsConstrained(const gfx::Rect& bounds) {
-  gfx::Rect work_area =
-      gfx::Screen::GetScreenFor(GetNativeView())->GetDisplayNearestPoint(
-          bounds.origin()).work_area();
+  gfx::Rect work_area = display::Screen::GetScreen()
+                            ->GetDisplayNearestPoint(bounds.origin())
+                            .work_area();
   if (work_area.IsEmpty()) {
     SetBounds(bounds);
   } else {
@@ -530,6 +522,14 @@ void Widget::SetVisibilityChangedAnimationsEnabled(bool value) {
   native_widget_->SetVisibilityChangedAnimationsEnabled(value);
 }
 
+void Widget::SetVisibilityAnimationDuration(const base::TimeDelta& duration) {
+  native_widget_->SetVisibilityAnimationDuration(duration);
+}
+
+void Widget::SetVisibilityAnimationTransition(VisibilityTransition transition) {
+  native_widget_->SetVisibilityAnimationTransition(transition);
+}
+
 Widget::MoveLoopResult Widget::RunMoveLoop(
     const gfx::Vector2d& drag_offset,
     MoveLoopSource source,
@@ -557,7 +557,7 @@ void Widget::StackBelow(gfx::NativeView native_view) {
   native_widget_->StackBelow(native_view);
 }
 
-void Widget::SetShape(gfx::NativeRegion shape) {
+void Widget::SetShape(SkRegion* shape) {
   native_widget_->SetShape(shape);
 }
 
@@ -599,7 +599,9 @@ bool Widget::IsClosed() const {
 }
 
 void Widget::Show() {
-  TRACE_EVENT0("views", "Widget::Show");
+  const ui::Layer* layer = GetLayer();
+  TRACE_EVENT1("views", "Widget::Show", "layer",
+               layer ? layer->name() : "none");
   if (non_client_view_) {
     // While initializing, the kiosk mode will go to full screen before the
     // widget gets shown. In that case we stay in full screen mode, regardless
@@ -652,10 +654,6 @@ bool Widget::IsActive() const {
   return native_widget_->IsActive();
 }
 
-void Widget::DisableInactiveRendering() {
-  SetInactiveRenderingDisabled(true);
-}
-
 void Widget::SetAlwaysOnTop(bool on_top) {
   native_widget_->SetAlwaysOnTop(on_top);
 }
@@ -702,14 +700,10 @@ bool Widget::IsFullscreen() const {
   return native_widget_->IsFullscreen();
 }
 
-void Widget::SetOpacity(unsigned char opacity) {
+void Widget::SetOpacity(float opacity) {
   native_widget_->SetOpacity(opacity);
 }
 
-void Widget::SetUseDragFrame(bool use_drag_frame) {
-  native_widget_->SetUseDragFrame(use_drag_frame);
-}
-
 void Widget::FlashFrame(bool flash) {
   native_widget_->FlashFrame(flash);
 }
@@ -726,12 +720,12 @@ bool Widget::IsVisible() const {
   return native_widget_->IsVisible();
 }
 
-ui::ThemeProvider* Widget::GetThemeProvider() const {
+const ui::ThemeProvider* Widget::GetThemeProvider() const {
   const Widget* root_widget = GetTopLevelWidget();
   if (root_widget && root_widget != this) {
     // Attempt to get the theme provider, and fall back to the default theme
     // provider if not found.
-    ui::ThemeProvider* provider = root_widget->GetThemeProvider();
+    const ui::ThemeProvider* provider = root_widget->GetThemeProvider();
     if (provider)
       return provider;
 
@@ -756,44 +750,44 @@ const FocusManager* Widget::GetFocusManager() const {
   return toplevel_widget ? toplevel_widget->focus_manager_.get() : NULL;
 }
 
-InputMethod* Widget::GetInputMethod() {
-  return const_cast<InputMethod*>(
-      const_cast<const Widget*>(this)->GetInputMethod());
-}
-
-const InputMethod* Widget::GetInputMethod() const {
+ui::InputMethod* Widget::GetInputMethod() {
   if (is_top_level()) {
-    if (!input_method_.get())
-      input_method_ = const_cast<Widget*>(this)->CreateInputMethod().Pass();
-    return input_method_.get();
+    // Only creates the shared the input method instance on top level widget.
+    return native_widget_private()->GetInputMethod();
   } else {
-    const Widget* toplevel = GetTopLevelWidget();
+    Widget* toplevel = GetTopLevelWidget();
     // If GetTopLevelWidget() returns itself which is not toplevel,
     // the widget is detached from toplevel widget.
     // TODO(oshima): Fix GetTopLevelWidget() to return NULL
     // if there is no toplevel. We probably need to add GetTopMostWidget()
     // to replace some use cases.
-    return (toplevel && toplevel != this) ? toplevel->GetInputMethod() : NULL;
+    return (toplevel && toplevel != this) ? toplevel->GetInputMethod()
+                                          : nullptr;
   }
 }
 
-ui::InputMethod* Widget::GetHostInputMethod() {
-  return native_widget_private()->GetHostInputMethod();
-}
-
 void Widget::RunShellDrag(View* view,
                           const ui::OSExchangeData& data,
                           const gfx::Point& location,
                           int operation,
                           ui::DragDropTypes::DragEventSource source) {
   dragged_view_ = view;
+  OnDragWillStart();
+
+  WidgetDeletionObserver widget_deletion_observer(this);
   native_widget_->RunShellDrag(view, data, location, operation, source);
+
+  // The widget may be destroyed during the drag operation.
+  if (!widget_deletion_observer.IsWidgetAlive())
+    return;
+
   // If the view is removed during the drag operation, dragged_view_ is set to
   // NULL.
   if (view && dragged_view_ == view) {
     dragged_view_ = NULL;
     view->OnDragDone();
   }
+  OnDragComplete();
 }
 
 void Widget::SchedulePaintInRect(const gfx::Rect& rect) {
@@ -852,6 +846,10 @@ void Widget::LocaleChanged() {
   root_view_->LocaleChanged();
 }
 
+void Widget::DeviceScaleFactorChanged(float device_scale_factor) {
+  root_view_->DeviceScaleFactorChanged(device_scale_factor);
+}
+
 void Widget::SetFocusTraversableParent(FocusTraversable* parent) {
   root_view_->SetFocusTraversableParent(parent);
 }
@@ -869,9 +867,9 @@ NonClientFrameView* Widget::CreateNonClientFrameView() {
       widget_delegate_->CreateNonClientFrameView(this);
   if (!frame_view)
     frame_view = native_widget_->CreateNonClientFrameView();
-  if (!frame_view && ViewsDelegate::views_delegate) {
+  if (!frame_view && ViewsDelegate::GetInstance()) {
     frame_view =
-        ViewsDelegate::views_delegate->CreateDefaultNonClientFrameView(this);
+        ViewsDelegate::GetInstance()->CreateDefaultNonClientFrameView(this);
   }
   if (frame_view)
     return frame_view;
@@ -910,11 +908,7 @@ const ui::Compositor* Widget::GetCompositor() const {
   return native_widget_->GetCompositor();
 }
 
-ui::Compositor* Widget::GetCompositor() {
-  return native_widget_->GetCompositor();
-}
-
-ui::Layer* Widget::GetLayer() {
+const ui::Layer* Widget::GetLayer() const {
   return native_widget_->GetLayer();
 }
 
@@ -938,13 +932,17 @@ NativeWidget* Widget::native_widget() {
 }
 
 void Widget::SetCapture(View* view) {
+  if (!native_widget_->HasCapture()) {
+    native_widget_->SetCapture();
+
+    // Early return if setting capture was unsuccessful.
+    if (!native_widget_->HasCapture())
+      return;
+  }
+
   if (internal::NativeWidgetPrivate::IsMouseButtonDown())
     is_mouse_button_pressed_ = true;
-  if (internal::NativeWidgetPrivate::IsTouchDown())
-    is_touch_down_ = true;
   root_view_->SetMouseHandler(view);
-  if (!native_widget_->HasCapture())
-    native_widget_->SetCapture();
 }
 
 void Widget::ReleaseCapture() {
@@ -969,10 +967,16 @@ gfx::Rect Widget::GetWorkAreaBoundsInScreen() const {
 }
 
 void Widget::SynthesizeMouseMoveEvent() {
+  // In screen coordinate.
+  gfx::Point mouse_location = EventMonitor::GetLastMouseLocation();
+  if (!GetWindowBoundsInScreen().Contains(mouse_location))
+    return;
+
+  // Convert: screen coordinate -> widget coordinate.
+  View::ConvertPointFromScreen(root_view_.get(), &mouse_location);
   last_mouse_event_was_move_ = false;
-  ui::MouseEvent mouse_event(ui::ET_MOUSE_MOVED,
-                             last_mouse_event_position_,
-                             last_mouse_event_position_,
+  ui::MouseEvent mouse_event(ui::ET_MOUSE_MOVED, mouse_location,
+                             mouse_location, ui::EventTimeForNow(),
                              ui::EF_IS_SYNTHESIZED, 0);
   root_view_->OnMouseMoved(mouse_event);
 }
@@ -985,7 +989,15 @@ bool Widget::IsTranslucentWindowOpacitySupported() const {
   return native_widget_->IsTranslucentWindowOpacitySupported();
 }
 
-void Widget::OnOwnerClosing() {
+void Widget::OnSizeConstraintsChanged() {
+  native_widget_->OnSizeConstraintsChanged();
+  non_client_view_->SizeConstraintsChanged();
+}
+
+void Widget::OnOwnerClosing() {}
+
+std::string Widget::GetName() const {
+  return native_widget_->GetName();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1003,12 +1015,8 @@ bool Widget::CanActivate() const {
   return widget_delegate_->CanActivate();
 }
 
-bool Widget::IsInactiveRenderingDisabled() const {
-  return disable_inactive_rendering_;
-}
-
-void Widget::EnableInactiveRendering() {
-  SetInactiveRenderingDisabled(false);
+bool Widget::IsAlwaysRenderAsActive() const {
+  return always_render_as_active_;
 }
 
 void Widget::OnNativeWidgetActivationChanged(bool active) {
@@ -1021,31 +1029,16 @@ void Widget::OnNativeWidgetActivationChanged(bool active) {
   FOR_EACH_OBSERVER(WidgetObserver, observers_,
                     OnWidgetActivationChanged(this, active));
 
-  // During window creation, the widget gets focused without activation, and in
-  // that case, the focus manager cannot set the appropriate text input client
-  // because the widget is not active.  Thus we have to notify the focus manager
-  // not only when the focus changes but also when the widget gets activated.
-  // See crbug.com/377479 for details.
-  views::FocusManager* focus_manager = GetFocusManager();
-  if (focus_manager) {
-    if (active)
-      focus_manager->FocusTextInputClient(focus_manager->GetFocusedView());
-    else
-      focus_manager->BlurTextInputClient(focus_manager->GetFocusedView());
-  }
-
-  if (IsVisible() && non_client_view())
-    non_client_view()->frame_view()->SchedulePaint();
+  if (non_client_view())
+    non_client_view()->frame_view()->ActivationChanged(active);
 }
 
-void Widget::OnNativeFocus(gfx::NativeView old_focused_view) {
-  WidgetFocusManager::GetInstance()->OnWidgetFocusEvent(old_focused_view,
-                                                        GetNativeView());
+void Widget::OnNativeFocus() {
+  WidgetFocusManager::GetInstance()->OnNativeFocusChanged(GetNativeView());
 }
 
-void Widget::OnNativeBlur(gfx::NativeView new_focused_view) {
-  WidgetFocusManager::GetInstance()->OnWidgetFocusEvent(GetNativeView(),
-                                                        new_focused_view);
+void Widget::OnNativeBlur() {
+  WidgetFocusManager::GetInstance()->OnNativeFocusChanged(nullptr);
 }
 
 void Widget::OnNativeWidgetVisibilityChanging(bool visible) {
@@ -1075,7 +1068,7 @@ void Widget::OnNativeWidgetCreated(bool desktop_widget) {
 void Widget::OnNativeWidgetDestroying() {
   // Tell the focus manager (if any) that root_view is being removed
   // in case that the focused view is under this root view.
-  if (GetFocusManager())
+  if (GetFocusManager() && root_view_)
     GetFocusManager()->ViewRemoved(root_view_.get());
   FOR_EACH_OBSERVER(WidgetObserver, observers_, OnWidgetDestroying(this));
   if (non_client_view_)
@@ -1100,12 +1093,8 @@ gfx::Size Widget::GetMaximumSize() const {
 
 void Widget::OnNativeWidgetMove() {
   widget_delegate_->OnWidgetMove();
-  View* root = GetRootView();
-  if (root && root->GetFocusManager()) {
-    View* focused_view = root->GetFocusManager()->GetFocusedView();
-    if (focused_view && focused_view->GetInputMethod())
-      focused_view->GetInputMethod()->OnCaretBoundsChanged(focused_view);
-  }
+  NotifyCaretBoundsChanged(GetInputMethod());
+
   FOR_EACH_OBSERVER(WidgetObserver, observers_, OnWidgetBoundsChanged(
     this,
     GetWindowBoundsInScreen()));
@@ -1113,25 +1102,23 @@ void Widget::OnNativeWidgetMove() {
 
 void Widget::OnNativeWidgetSizeChanged(const gfx::Size& new_size) {
   View* root = GetRootView();
-  if (root) {
+  if (root)
     root->SetSize(new_size);
-    if (root->GetFocusManager()) {
-      View* focused_view = GetRootView()->GetFocusManager()->GetFocusedView();
-      if (focused_view && focused_view->GetInputMethod())
-        focused_view->GetInputMethod()->OnCaretBoundsChanged(focused_view);
-    }
-  }
-  // Size changed notifications can fire prior to full initialization
-  // i.e. during session restore.  Avoid saving session state during these
-  // startup procedures.
-  if (native_widget_initialized_)
-    SaveWindowPlacement();
+
+  NotifyCaretBoundsChanged(GetInputMethod());
+  SaveWindowPlacementIfInitialized();
 
   FOR_EACH_OBSERVER(WidgetObserver, observers_, OnWidgetBoundsChanged(
     this,
     GetWindowBoundsInScreen()));
 }
 
+void Widget::OnNativeWidgetWorkspaceChanged() {}
+
+void Widget::OnNativeWidgetWindowShowStateChanged() {
+  SaveWindowPlacementIfInitialized();
+}
+
 void Widget::OnNativeWidgetBeginUserBoundsChange() {
   widget_delegate_->OnWindowBeginUserBoundsChange();
 }
@@ -1144,20 +1131,12 @@ bool Widget::HasFocusManager() const {
   return !!focus_manager_.get();
 }
 
-bool Widget::OnNativeWidgetPaintAccelerated(const gfx::Rect& dirty_region) {
-  ui::Compositor* compositor = GetCompositor();
-  if (!compositor)
-    return false;
-
-  compositor->ScheduleRedrawRect(dirty_region);
-  return true;
-}
-
-void Widget::OnNativeWidgetPaint(gfx::Canvas* canvas) {
+void Widget::OnNativeWidgetPaint(const ui::PaintContext& context) {
   // On Linux Aura, we can get here during Init() because of the
   // SetInitialBounds call.
-  if (native_widget_initialized_)
-    GetRootView()->Paint(canvas, CullSet());
+  if (!native_widget_initialized_)
+    return;
+  GetRootView()->Paint(context);
 }
 
 int Widget::GetNonClientComponent(const gfx::Point& point) {
@@ -1173,6 +1152,10 @@ int Widget::GetNonClientComponent(const gfx::Point& point) {
 
 void Widget::OnKeyEvent(ui::KeyEvent* event) {
   SendEventToProcessor(event);
+  if (!event->handled() && GetFocusManager() &&
+      !GetFocusManager()->OnKeyEvent(*event)) {
+    event->StopPropagation();
+  }
 }
 
 // TODO(tdanderson): We should not be calling the OnMouse*() functions on
@@ -1188,19 +1171,23 @@ void Widget::OnMouseEvent(ui::MouseEvent* event) {
       // use an observer to make sure we are still alive.
       WidgetDeletionObserver widget_deletion_observer(this);
 
+      gfx::NativeView current_capture =
+          internal::NativeWidgetPrivate::GetGlobalCapture(
+              native_widget_->GetNativeView());
       // Make sure we're still visible before we attempt capture as the mouse
       // press processing may have made the window hide (as happens with menus).
-
-      // It is possible for a View to show a context menu on mouse-press. Since
-      // the menu does a capture and starts a nested message-loop, the release
-      // would go to the menu. The next click (i.e. both mouse-press and release
-      // events) also go to the menu. The menu (and the nested message-loop)
-      // gets closed after this second release event. The code then resumes from
-      // here. So make sure that the mouse-button is still down before doing a
-      // capture.
+      //
+      // It is possible that capture has changed as a result of a mouse-press.
+      // In these cases do not update internal state.
+      //
+      // A mouse-press may trigger a nested message-loop, and absorb the paired
+      // release. If so the code returns here. So make sure that that
+      // mouse-button is still down before attempting to do a capture.
       if (root_view && root_view->OnMousePressed(*event) &&
           widget_deletion_observer.IsWidgetAlive() && IsVisible() &&
-          internal::NativeWidgetPrivate::IsMouseButtonDown()) {
+          internal::NativeWidgetPrivate::IsMouseButtonDown() &&
+          current_capture == internal::NativeWidgetPrivate::GetGlobalCapture(
+                                 native_widget_->GetNativeView())) {
         is_mouse_button_pressed_ = true;
         if (!native_widget_->HasCapture())
           native_widget_->SetCapture();
@@ -1213,8 +1200,10 @@ void Widget::OnMouseEvent(ui::MouseEvent* event) {
       last_mouse_event_was_move_ = false;
       is_mouse_button_pressed_ = false;
       // Release capture first, to avoid confusion if OnMouseReleased blocks.
-      if (auto_release_capture_ && native_widget_->HasCapture())
+      if (auto_release_capture_ && native_widget_->HasCapture()) {
+        base::AutoReset<bool> resetter(&ignore_capture_loss_, true);
         native_widget_->ReleaseCapture();
+      }
       if (root_view)
         root_view->OnMouseReleased(*event);
       if ((event->flags() & ui::EF_IS_NON_CLIENT) == 0)
@@ -1254,12 +1243,12 @@ void Widget::OnMouseEvent(ui::MouseEvent* event) {
 }
 
 void Widget::OnMouseCaptureLost() {
-  if (is_mouse_button_pressed_ || is_touch_down_) {
-    View* root_view = GetRootView();
-    if (root_view)
-      root_view->OnMouseCaptureLost();
-  }
-  is_touch_down_ = false;
+  if (ignore_capture_loss_)
+    return;
+
+  View* root_view = GetRootView();
+  if (root_view)
+    root_view->OnMouseCaptureLost();
   is_mouse_button_pressed_ = false;
 }
 
@@ -1275,22 +1264,9 @@ void Widget::OnScrollEvent(ui::ScrollEvent* event) {
 }
 
 void Widget::OnGestureEvent(ui::GestureEvent* event) {
-  switch (event->type()) {
-    case ui::ET_GESTURE_TAP_DOWN:
-      is_touch_down_ = true;
-      // We explicitly don't capture here. Not capturing enables multiple
-      // widgets to get tap events at the same time. Views (such as tab
-      // dragging) may explicitly capture.
-      break;
-
-    case ui::ET_GESTURE_END:
-      if (event->details().touch_points() == 1)
-        is_touch_down_ = false;
-      break;
-
-    default:
-      break;
-  }
+  // We explicitly do not capture here. Not capturing enables multiple widgets
+  // to get tap events at the same time. Views (such as tab dragging) may
+  // explicitly capture.
   SendEventToProcessor(event);
 }
 
@@ -1298,10 +1274,6 @@ bool Widget::ExecuteCommand(int command_id) {
   return widget_delegate_->ExecuteWindowsCommand(command_id);
 }
 
-InputMethod* Widget::GetInputMethodDirect() {
-  return input_method_.get();
-}
-
 const std::vector<ui::Layer*>& Widget::GetRootLayers() {
   if (root_layers_dirty_) {
     root_layers_dirty_ = false;
@@ -1334,7 +1306,7 @@ bool Widget::SetInitialFocus(ui::WindowShowState show_state) {
       show_state == ui::SHOW_STATE_MINIMIZED) {
     // If not focusing the window now, tell the focus manager which view to
     // focus when the window is restored.
-    if (v)
+    if (v && focus_manager_.get())
       focus_manager_->SetStoredFocusView(v);
     return true;
   }
@@ -1395,20 +1367,26 @@ internal::RootView* Widget::CreateRootView() {
 void Widget::DestroyRootView() {
   non_client_view_ = NULL;
   root_view_.reset();
-  // Input method has to be destroyed before focus manager.
-  input_method_.reset();
 }
 
+void Widget::OnDragWillStart() {}
+
+void Widget::OnDragComplete() {}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Widget, private:
 
-void Widget::SetInactiveRenderingDisabled(bool value) {
-  if (value == disable_inactive_rendering_)
+void Widget::SetAlwaysRenderAsActive(bool always_render_as_active) {
+  if (always_render_as_active_ == always_render_as_active)
     return;
 
-  disable_inactive_rendering_ = value;
-  if (non_client_view_)
-    non_client_view_->SetInactiveRenderingDisabled(value);
+  always_render_as_active_ = always_render_as_active;
+
+  // If active, the frame should already be painted. Otherwise,
+  // |always_render_as_active_| just changed, and the widget is inactive, so
+  // schedule a repaint.
+  if (non_client_view_ && !IsActive())
+    non_client_view_->frame_view()->SchedulePaint();
 }
 
 void Widget::SaveWindowPlacement() {
@@ -1425,6 +1403,11 @@ void Widget::SaveWindowPlacement() {
   widget_delegate_->SaveWindowPlacement(bounds, show_state);
 }
 
+void Widget::SaveWindowPlacementIfInitialized() {
+  if (native_widget_initialized_)
+    SaveWindowPlacement();
+}
+
 void Widget::SetInitialBounds(const gfx::Rect& bounds) {
   if (!non_client_view_)
     return;
@@ -1441,9 +1424,16 @@ void Widget::SetInitialBounds(const gfx::Rect& bounds) {
     }
   } else {
     if (bounds.IsEmpty()) {
-      // No initial bounds supplied, so size the window to its content and
-      // center over its parent.
-      native_widget_->CenterWindow(non_client_view_->GetPreferredSize());
+      if (bounds.origin().IsOrigin()) {
+        // No initial bounds supplied, so size the window to its content and
+        // center over its parent.
+        native_widget_->CenterWindow(non_client_view_->GetPreferredSize());
+      } else {
+        // Use the preferred size and the supplied origin.
+        gfx::Rect preferred_bounds(bounds);
+        preferred_bounds.set_size(non_client_view_->GetPreferredSize());
+        SetBoundsConstrained(preferred_bounds);
+      }
     } else {
       // Use the supplied initial bounds.
       SetBoundsConstrained(bounds);
@@ -1493,19 +1483,6 @@ bool Widget::GetSavedWindowPlacement(gfx::Rect* bounds,
   return false;
 }
 
-scoped_ptr<InputMethod> Widget::CreateInputMethod() {
-  scoped_ptr<InputMethod> input_method(native_widget_->CreateInputMethod());
-  if (input_method.get())
-    input_method->Init(this);
-  return input_method.Pass();
-}
-
-void Widget::ReplaceInputMethod(InputMethod* input_method) {
-  input_method_.reset(input_method);
-  input_method->SetDelegate(native_widget_->GetInputMethodDelegate());
-  input_method->Init(this);
-}
-
 namespace internal {
 
 ////////////////////////////////////////////////////////////////////////////////
